using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Reflection;
using System.Transactions;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.Attributes;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.Resources;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.AnovoRx.Data.Generators.BarcodeController;


namespace AddUpdateScript
{
    class Program
    {
		static string CONNECTION_STRING = "Server=localhost;CompanyDB=SBODemoUS;ExtraDB=PMX_SBODemoUS;UseTrusted=False;DbUserName=sa;DbPassword=B1Admin;UserName=manager;Password=produmex;Pooling=True;MinPoolSize=2;MaxPoolSize=10;ServerType=MSSQL2008;DBCultureInfo=en-US;LicenseServer=localhost:30000";
		/// <summary>
        /// Used for logging
        /// </summary>
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        static void Main(string[] args)
        {
            try
            {
                Produmex.Foundation.Diagnostics.LogProvider.SetConsoleAppender("WARN");
                Execute();
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
            }
            if (Debugger.IsAttached)
            {
                Console.WriteLine("Press <Enter>");
                Console.ReadLine();
            }
        }

        static void Execute()
        {
            Console.WriteLine( "Extensions ..." );

            using ( TransactionScope scope = PmxDbConnection.GetNewTransactionScope() )
            {
                using ( PmxDbConnection conn = PmxDbConnectionMgr.GetDirectConnection( SboConnectionString.ParseStringToObject( CONNECTION_STRING ) ) )
                {
                    conn.Open();

                    // bar code controller
                    AddUpdateExtension(
                         AnovoRxBarcodeController.IDs.ExtCode,
						 "AnovoRx Bar Code Controller",
                         typeof(AnovoRxBarcodeController),
                         Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionTypeCodes.IBARCODECONTROLLER,
                         conn
                         );

                    scope.Complete();
                }
            }

            Console.WriteLine( "Done ..." );
        }


        /// <summary>
        /// Add/update <see cref="PmxExtensionType"/>
        /// </summary>
        /// <param name="code">The code of the extension type.</param>
        /// <param name="name">The name of the extension type.</param>
        /// <param name="interfaceType">The interface type of the extension type.</param>
        /// <param name="conn">The DB connection</param>
        public static void AddUpdateExtensionType( string code, string name, Type interfaceType, PmxDbConnection conn )
        {
            PmxExtensionTypeProvider prov = new PmxExtensionTypeProvider( conn );
            PmxExtensionType exType = null;
            try
            {
                bool newBO = false;
                exType = prov.GetBO( code );
                if ( exType == null )
                {
                    newBO = true;
                    exType = prov.GetNewBO();
                    exType.Code = code;
                }
                exType.Name = name;
                exType.InterfaceType = ReflectionHelper.GetTypeAndAssemblyName( interfaceType );
                if ( newBO )
                {
                    prov.AddBO( exType );
                }
                else
                {
                    prov.UpdateBO( exType, false, false );
                }
            }
            finally
            {
                if ( exType != null ) exType.Dispose();
            }
        }

        /// <summary>
        /// Add/Update <see cref="PmxExtension"/>
        /// </summary>
        /// <param name="code">The code of the extension.</param>
        /// <param name="name">The name of the extension.</param>
        /// <param name="type">The type of the extension.</param>
        /// <param name="extensionTypeCode">The extension type code.</param>
        /// <param name="conn">The DB connection</param>
        private static void AddUpdateExtension( string code, string name, Type type, string extensionTypeCode, PmxDbConnection conn )
        {
            PmxExtensionProvider prov = new PmxExtensionProvider( conn );
            PmxExtension extension = null;
            try
            {
                bool newBO = false;
                extension = prov.GetBO( code );
                if ( extension == null )
                {
                    newBO = true;
                    extension = prov.GetNewBO();
                    extension.Code = code;
                }
                extension.Name = name;
                extension.ExtensionTypeCode = extensionTypeCode;
                extension.Type = ReflectionHelper.GetTypeAndAssemblyName( type );
                if ( newBO )
                {
                    prov.AddBO( extension );
                }
                else
                {
                    prov.UpdateBO( extension, false, false );
                }
            }
            finally
            {
                if ( extension != null ) extension.Dispose();
            }
        }

        /// <summary>
        /// Add/Update <see cref="PmxProperty"/>
        /// </summary>
        /// <param name="code">The code of the property.</param>
        /// <param name="name">The name of the property.</param>
        /// <param name="extensionTypeCode">The extension type code.</param>
        /// <param name="conn">The DB connection</param>
        private static void AddUpdateProperty( string code, string name, string extensionTypeCode, PmxDbConnection conn )
        {
            AddUpdateProperty( code, name, extensionTypeCode, null, conn );
        }


        /// <summary>
        /// Add/Update <see cref="PmxProperty"/>
        /// </summary>
        /// <param name="code">The code of the property.</param>
        /// <param name="name">The name of the property.</param>
        /// <param name="extensionTypeCode">The extension type code.</param>
        /// <param name="defaultNewExtensionCode">The default new extension code.</param>
        /// <param name="conn">The DB connection</param>
        public static void AddUpdateProperty( string code, string name, string extensionTypeCode, string defaultNewExtensionCode, PmxDbConnection conn )
        {
            PmxPropertyProvider prov = new PmxPropertyProvider( conn );
            PmxProperty property = null;
            try
            {
                bool newBO = false;
                property = prov.GetBO( code );
                if ( property == null )
                {
                    newBO = true;
                    property = prov.GetNewBO();
                    property.Code = code;
                }
                property.Name = name;
                property.ExtensionTypeCode = extensionTypeCode;

                if ( newBO )
                {
                    if ( !string.IsNullOrEmpty( defaultNewExtensionCode ) )
                    {
                        //only set when defaulExtansionCode is set, and the property is new
                        property.ExtensionCode = defaultNewExtensionCode;
                    }
                    prov.AddBO( property );
                }
                else
                {
                    prov.UpdateBO( property, false, false );
                }
            }
            finally
            {
                if ( property != null ) property.Dispose();
            }
        }

        /// <summary>
        /// Add/Update <see cref="PmxExtensionParameter"/>
        /// </summary>
        /// <param name="code">The code of the extension parameter.</param>
        /// <param name="name">The name of the extension parameter.</param>
        /// <param name="extensionCode">The extension code.</param>
        /// <param name="convertorCode">The convertor code.</param>
        /// <param name="defaultValue">The default value.</param>
        /// <param name="conn">The DB connection</param>
        public static void AddUpdateExtensionParameter( string code, string name, string extensionCode, string convertorCode, string defaultValue, PmxDbConnection conn )
        {
            AddUpdateExtensionParameter( code, name, extensionCode, convertorCode, defaultValue, code, conn );
        }

        /// <summary>
        /// Add/Update <see cref="PmxExtensionParameter"/>
        /// </summary>
        /// <param name="code">The code of the extension parameter.</param>
        /// <param name="name">The name of the extension parameter.</param>
        /// <param name="extensionCode">The extension code.</param>
        /// <param name="convertorCode">The convertor code.</param>
        /// <param name="defaultValue">The default value.</param>
        /// <param name="generalParameterCode">The general parameter code.</param>
        /// <param name="conn">The DB connection</param>
        public static void AddUpdateExtensionParameter( string code, string name, string extensionCode, string convertorCode, string defaultValue, string generalParameterCode, PmxDbConnection conn )
        {
            AddUpdateExtensionParameter( code, name, extensionCode, convertorCode, defaultValue, generalParameterCode, 0, conn );
        }

        /// <summary>
        /// Add/Update <see cref="PmxExtensionParameter"/>
        /// </summary>
        /// <param name="code">The code of the extension parameter.</param>
        /// <param name="name">The name of the extension parameter.</param>
        /// <param name="extensionCode">The extension code.</param>
        /// <param name="convertorCode">The convertor code.</param>
        /// <param name="defaultValue">The default value.</param>
        /// <param name="generalParameterCode">The general parameter code.</param>
        /// <param name="sequence">The sequence.</param>
        /// <param name="conn">The DB connection</param>
        public static void AddUpdateExtensionParameter( string code, string name, string extensionCode, string convertorCode, string defaultValue, string generalParameterCode, int sequence, PmxDbConnection conn )
        {
            PmxExtensionParameterProvider prov = new PmxExtensionParameterProvider( conn );
            PmxExtensionParameter extParam = null;
            try
            {
                bool newBO = false;
                extParam = prov.GetBO( code );
                if ( extParam == null )
                {
                    newBO = true;
                    extParam = prov.GetNewBO();
                    extParam.Code = code;
                }
                extParam.Name = name;
                extParam.ExtensionCode = extensionCode;
                extParam.ConvertorCode = convertorCode;
                extParam.GeneralParameterCode = generalParameterCode;
                extParam.DefaultValue = defaultValue;
                extParam.Sequence = sequence;
                if ( newBO )
                {
                    prov.AddBO( extParam );
                }
                else
                {
                    prov.UpdateBO( extParam, false, false );
                }
            }
            finally
            {
                if ( extParam != null ) extParam.Dispose();
            }
        }

        /// <summary>
        /// Add <see cref="PmxParameterValidValues"/>
        /// </summary>
        /// <param name="parameterCode">The parameter code.</param>
        /// <param name="parameterValidValue">The parameter valid value.</param>
        /// <param name="parameterValidValueDescription">The parameter valid value description.</param>
        /// <param name="conn">The DB connection</param>
        public static void AddUpdateParameterValidValue( string parameterCode, string parameterValidValue, string parameterValidValueDescription, PmxDbConnection conn )
        {
            PmxParameterValidValuesProvider prov = new PmxParameterValidValuesProvider( conn );
            PmxParameterValidValues validValue = null;
            try
            {
                bool newBO = false;
                validValue = prov.GetParameterValidValue( parameterCode, parameterValidValue );
                if ( validValue == null )
                {
                    newBO = true;
                    validValue = prov.GetNewBO();
                    validValue.ParameterCode = parameterCode;
                    validValue.ParameterValidValue = parameterValidValue;
                }
                validValue.ParameterValidValueDescription = parameterValidValueDescription;
                if ( newBO )
                {
                    prov.AddBO( validValue );
                }
                else
                {
                    prov.UpdateBO( validValue, false, false );
                }
            }
            finally
            {
                if ( validValue != null ) validValue.Dispose();
            }

        }

        /// <summary>
        /// Adds/updates parameter valid values.
        /// </summary>
        /// <param name="parameterCode">The parameter code.</param>
        /// <param name="staticClassType">Type of the static class.</param>
        /// <param name="conn">The DB connection</param>
        public static void AddUpdateParameterValidValues( string parameterCode, Type staticClassType, PmxDbConnection conn )
        {
            FieldInfo[] fis = staticClassType.GetFields();
            foreach ( FieldInfo fi in fis )
            {
                foreach ( DescriptionAttribute attr in fi.GetCustomAttributes( typeof( DescriptionAttribute ), false ) )
                {
                    AddUpdateParameterValidValue( parameterCode, (string)fi.GetValue( null ), attr.Description, conn );
                }
            }
        }

        /// <summary>
        /// Preprocesses the SQL script.
        /// </summary>
        /// <param name="query">The query.</param>
        /// <returns></returns>
        private static string PreprocessSqlScript( string query, PmxDbConnection conn )
        {
            string serverType;
            switch ( conn.ConnectionStringObject.ServerType )
            {
                case SboServerType.HANA:
                    serverType = "HANA";
                    query = query.Replace( "{DB-SCHEMA}", "\"" + conn.ConnectionStringObject.CompanyDBSchema.SchemaName + "\"" );
                    break;
                default:
                    serverType = "MSSQL";
                    break;
            }

            int mssqlBlockBegin = query.IndexOf( "--<?DB:" );
            while ( mssqlBlockBegin >= 0 )
            {
                int mssqlBlockEnd = query.IndexOf( "--?>", mssqlBlockBegin );
                if ( mssqlBlockEnd < 0 )
                {
                    break;
                }

                if ( query.Substring( mssqlBlockBegin + "--<?DB:".Length, serverType.Length ) == serverType )
                {
                    query = query.Substring( 0, mssqlBlockBegin )
                          + query.Substring( mssqlBlockBegin + "--<?DB:".Length + serverType.Length, mssqlBlockEnd - mssqlBlockBegin - "--<?DB:".Length - serverType.Length )
                          + query.Substring( mssqlBlockEnd + "--?>".Length );
                    mssqlBlockEnd -= "--<?DB:".Length + serverType.Length + "--?>".Length;
                }
                else
                {
                    query = query.Substring( 0, mssqlBlockBegin ) + query.Substring( mssqlBlockEnd + "--?>".Length );
                    mssqlBlockEnd = mssqlBlockBegin;
                }
                mssqlBlockBegin = query.IndexOf( "--<?DB:", mssqlBlockEnd );
            }

            if ( conn.ConnectionStringObject.ServerType == SboServerType.HANA )
            {
                int tokenPos = query.IndexOf( "ISNULL(", StringComparison.OrdinalIgnoreCase );
                while ( tokenPos >= 0 )
                {
                    int endPos = tokenPos + "ISNULL(".Length;
                    query = query.Substring( 0, tokenPos ) + "IFNULL(" + query.Substring( endPos );
                    tokenPos = query.IndexOf( "ISNULL(", endPos, StringComparison.OrdinalIgnoreCase );
                }

                tokenPos = query.IndexOf( "@" );
                while ( tokenPos >= 0 )
                {
                    if ( query[tokenPos - 1] != '"' && Char.IsLetter( query[tokenPos + 1] ) )
                    {
                        query = query.Substring( 0, tokenPos ) + "p_" + query.Substring( tokenPos + 1 );
                    }
                    tokenPos = query.IndexOf( "@", tokenPos + 2 );
                }
            }

            return query;
        }
    }
}
